@@ -84,7 +84,7 @@ class Command(BaseCommand):
         parser.add_argument('--key-file', dest='key_file_path', action="store", type=str,
                             help='SSL .key file path. If not provided path from --cert-file will be selected. '
                                  'Either --cert-file or --key-file must be provided to use SSL.')
-        parser.add_argument('--extra-file', dest='extra_files', action="append", type=str,
+        parser.add_argument('--extra-file', dest='extra_files', action="append", type=str, default=[],
                             help='auto-reload whenever the given file changes too (can be specified multiple times)')
         parser.add_argument('--reloader-interval', dest='reloader_interval', action="store", type=int, default=DEFAULT_POLLER_RELOADER_INTERVAL,
                             help='After how many seconds auto-reload should scan for updates in poller-mode [default=%s]' % DEFAULT_POLLER_RELOADER_INTERVAL)
@@ -113,8 +113,8 @@ class Command(BaseCommand):
 
     @signalcommand
     def handle(self, *args, **options):
-        addrport = options.get('addrport')
-        startup_messages = options.get('startup_messages', 'reload')
+        addrport = options['addrport']
+        startup_messages = options['startup_messages']
         if startup_messages == "reload":
             self.show_startup_messages = os.environ.get('RUNSERVER_PLUS_SHOW_MESSAGES')
         elif startup_messages == "once":
@@ -130,7 +130,7 @@ class Command(BaseCommand):
         if hasattr(self.stderr, 'ending'):
             self.stderr.ending = None
 
-        setup_logger(logger, self.stderr, filename=options.get('output_file', None))  # , fmt="[%(name)s] %(message)s")
+        setup_logger(logger, self.stderr, filename=options['output_file'])  # , fmt="[%(name)s] %(message)s")
         logredirect = RedirectHandler(__name__)
 
         # Redirect werkzeug log items
@@ -139,7 +139,7 @@ class Command(BaseCommand):
         werklogger.addHandler(logredirect)
         werklogger.propagate = False
 
-        if options.get("print_sql", False):
+        if options["print_sql"]:
             try:
                 import sqlparse
             except ImportError:
@@ -180,9 +180,9 @@ class Command(BaseCommand):
 
             utils.CursorDebugWrapper = PrintQueryWrapper
 
-        pdb_option = options.get('pdb', False)
-        ipdb_option = options.get('ipdb', False)
-        pm = options.get('pm', False)
+        pdb_option = options['pdb']
+        ipdb_option = options['ipdb']
+        pm = options['pm']
         try:
             from django_pdb.middleware import PdbMiddleware
         except ImportError:
@@ -222,7 +222,7 @@ class Command(BaseCommand):
         from django.views import debug
         debug.technical_500_response = postmortem if pm else null_technical_500_response
 
-        self.use_ipv6 = options.get('use_ipv6')
+        self.use_ipv6 = options['use_ipv6']
         if self.use_ipv6 and not socket.has_ipv6:
             raise CommandError('Your Python does not support IPv6.')
         self._raw_ipv6 = False
@@ -277,19 +277,19 @@ class Command(BaseCommand):
         class WSGIRequestHandler(_WSGIRequestHandler):
             def make_environ(self):
                 environ = super(WSGIRequestHandler, self).make_environ()
-                if not options.get('keep_meta_shutdown_func'):
+                if not options['keep_meta_shutdown_func']:
                     del environ['werkzeug.server.shutdown']
                 return environ
 
-        threaded = options.get('threaded', True)
-        use_reloader = options.get('use_reloader', True)
-        open_browser = options.get('open_browser', False)
+        threaded = options['threaded']
+        use_reloader = options['use_reloader']
+        open_browser = options['open_browser']
         quit_command = (sys.platform == 'win32') and 'CTRL-BREAK' or 'CONTROL-C'
-        extra_files = options.get('extra_files', None) or []
-        reloader_interval = options.get('reloader_interval', 1)
-        reloader_type = options.get('reloader_type', 'auto')
+        extra_files = options['extra_files']
+        reloader_interval = options['reloader_interval']
+        reloader_type = options['reloader_type']
 
-        self.nopin = options.get('nopin', False)
+        self.nopin = options['nopin']
 
         if self.show_startup_messages:
             print("Performing system checks...\n")
@@ -303,11 +303,11 @@ class Command(BaseCommand):
             pass
         handler = get_internal_wsgi_application()
         if USE_STATICFILES:
-            use_static_handler = options.get('use_static_handler', True)
-            insecure_serving = options.get('insecure_serving', False)
+            use_static_handler = options['use_static_handler']
+            insecure_serving = options['insecure_serving']
             if use_static_handler and (settings.DEBUG or insecure_serving):
                 handler = StaticFilesHandler(handler)
-        if options.get("cert_path") or options.get("key_file_path"):
+        if options["cert_path"] or options["key_file_path"]:
             """
             OpenSSL is needed for SSL support.
 
@@ -412,8 +412,8 @@ class Command(BaseCommand):
 
     @classmethod
     def determine_ssl_files_paths(cls, options):
-        cert_file = cls._determine_path_for_file(options.get('cert_path'), options.get('key_file_path'), "crt")
-        key_file = cls._determine_path_for_file(options.get('key_file_path'), options.get('cert_path'), "key")
+        cert_file = cls._determine_path_for_file(options['cert_path'], options['key_file_path'], "crt")
+        key_file = cls._determine_path_for_file(options['key_file_path'], options['cert_path'], "key")
         return cert_file, key_file
 
     if django.VERSION[:2] <= (1, 9):
