@@ -26,56 +26,83 @@ def use_vi_mode():
 class Command(BaseCommand):
     help = "Like the 'shell' command but autoloads the models of all installed Django apps."
     extra_args = None
+    tests_mode = False
 
     def add_arguments(self, parser):
         super(Command, self).add_arguments(parser)
         parser.add_argument(
             '--plain', action='store_true', dest='plain',
-            help='Tells Django to use plain Python, not BPython nor IPython.')
+            default=False,
+            help='Tells Django to use plain Python, not BPython nor IPython.'
+        )
         parser.add_argument(
             '--bpython', action='store_true', dest='bpython',
-            help='Tells Django to use BPython, not IPython.')
+            default=False,
+            help='Tells Django to use BPython, not IPython.'
+        )
         parser.add_argument(
             '--ptpython', action='store_true', dest='ptpython',
-            help='Tells Django to use PTPython, not IPython.')
+            default=False,
+            help='Tells Django to use PTPython, not IPython.'
+        )
         parser.add_argument(
             '--ptipython', action='store_true', dest='ptipython',
-            help='Tells Django to use PT-IPython, not IPython.')
+            default=False,
+            help='Tells Django to use PT-IPython, not IPython.'
+        )
         parser.add_argument(
             '--ipython', action='store_true', dest='ipython',
-            help='Tells Django to use IPython, not BPython.')
+            default=False,
+            help='Tells Django to use IPython, not BPython.'
+        )
         parser.add_argument(
             '--notebook', action='store_true', dest='notebook',
-            help='Tells Django to use IPython Notebook.')
+            default=False,
+            help='Tells Django to use IPython Notebook.'
+        )
         parser.add_argument(
             '--kernel', action='store_true', dest='kernel',
-            help='Tells Django to start an IPython Kernel.')
+            default=False,
+            help='Tells Django to start an IPython Kernel.'
+        )
         parser.add_argument(
             '--connection-file', action='store', dest='connection_file',
-            help='Specifies the connection file to use if using the --kernel option'),
+            help='Specifies the connection file to use if using the --kernel option'
+        )
         parser.add_argument(
             '--no-startup', action='store_true', dest='no_startup',
-            help='When using plain Python, ignore the PYTHONSTARTUP environment variable and ~/.pythonrc.py script.')
+            default=False,
+            help='When using plain Python, ignore the PYTHONSTARTUP environment variable and ~/.pythonrc.py script.'
+        )
         parser.add_argument(
             '--use-pythonrc', action='store_true', dest='use_pythonrc',
-            help='When using plain Python, load the PYTHONSTARTUP environment variable and ~/.pythonrc.py script.')
+            default=False,
+            help='When using plain Python, load the PYTHONSTARTUP environment variable and ~/.pythonrc.py script.'
+        )
         parser.add_argument(
-            '--print-sql', action='store_true', default=False,
-            help="Print SQL queries as they're executed")
+            '--print-sql', action='store_true',
+            default=False,
+            help="Print SQL queries as they're executed"
+        )
         parser.add_argument(
             '--dont-load', action='append', dest='dont_load', default=[],
-            help='Ignore autoloading of some apps/models. Can be used '
-            'several times.')
+            help='Ignore autoloading of some apps/models. Can be used several times.'
+        )
         parser.add_argument(
-            '--quiet-load', action='store_true', default=False,
-            dest='quiet_load', help='Do not display loaded models messages')
+            '--quiet-load', action='store_true',
+            default=False,
+            dest='quiet_load', help='Do not display loaded models messages'
+        )
         parser.add_argument(
             '--vi', action='store_true', default=use_vi_mode(), dest='vi_mode',
-            help='Load Vi key bindings (for --ptpython and --ptipython)')
+            help='Load Vi key bindings (for --ptpython and --ptipython)'
+        )
         parser.add_argument(
-            '--no-browser', action='store_true', default=False,
+            '--no-browser', action='store_true',
+            default=False,
             dest='no_browser',
-            help='Don\'t open the notebook in a browser after startup.')
+            help='Don\'t open the notebook in a browser after startup.'
+        )
 
     def run_from_argv(self, argv):
         if '--' in argv[2:]:
@@ -103,7 +130,11 @@ class Command(BaseCommand):
         return arguments
 
     def get_imported_objects(self, options):
-        return import_objects(options, self.style)
+        imported_objects = import_objects(options, self.style)
+        if self.tests_mode:
+            # save imported objects so we can run tests against it later
+            self.tests_imported_objects = imported_objects
+        return imported_objects
 
     def get_kernel(self, options):
         try:
@@ -121,7 +152,7 @@ class Command(BaseCommand):
                 argv=[],
                 user_ns=imported_objects,
             )
-            connection_file = options.get('connection_file')
+            connection_file = options['connection_file']
             if connection_file:
                 kwargs['connection_file'] = connection_file
             start_kernel(**kwargs)
@@ -143,7 +174,7 @@ class Command(BaseCommand):
                 except ImportError:
                     return traceback.format_exc()
 
-        no_browser = options.get('no_browser', False)
+        no_browser = options['no_browser']
 
         def install_kernel_spec(app, display_name, ipython_arguments):
             """install an IPython >= 3.0 kernelspec that loads django extensions"""
@@ -225,10 +256,16 @@ class Command(BaseCommand):
             # we already know 'readline' was imported successfully.
             import rlcompleter
             readline.set_completer(rlcompleter.Completer(imported_objects).complete)
-            readline.parse_and_bind("tab:complete")
+            # Enable tab completion on systems using libedit (e.g. macOS).
+            # These lines are copied from Lib/site.py on Python 3.4.
+            readline_doc = getattr(readline, '__doc__', '')
+            if readline_doc is not None and 'libedit' in readline_doc:
+                readline.parse_and_bind("bind ^I rl_complete")
+            else:
+                readline.parse_and_bind("tab:complete")
 
-        use_pythonrc = options.get('use_pythonrc', False)
-        no_startup = options.get('no_startup', False)
+        use_pythonrc = options['use_pythonrc']
+        no_startup = options['no_startup']
 
         # We want to honor both $PYTHONSTARTUP and .pythonrc.py, so follow system
         # conventions and get $PYTHONSTARTUP first then .pythonrc.py.
@@ -238,13 +275,16 @@ class Command(BaseCommand):
                     continue
                 if not os.path.isfile(pythonrc):
                     continue
+                with open(pythonrc) as handle:
+                    pythonrc_code = handle.read()
                 # Match the behavior of the cpython shell where an error in
                 # PYTHONSTARTUP prints an exception and continues.
                 try:
-                    with open(pythonrc) as handle:
-                        exec(compile(handle.read(), pythonrc, 'exec'), imported_objects) in globals(), locals()
+                    exec(compile(pythonrc_code, pythonrc, 'exec'), imported_objects)
                 except Exception:
                     traceback.print_exc()
+                    if self.tests_mode:
+                        raise
 
         def run_plain():
             code.interact(local=imported_objects)
@@ -304,7 +344,7 @@ class Command(BaseCommand):
             imported_objects = self.get_imported_objects(options)
             history_filename = os.path.expanduser('~/.ptpython_history')
             embed(globals=imported_objects, history_filename=history_filename,
-                  vi_mode=options.get('vi_mode', False), configure=run_config)
+                  vi_mode=options['vi_mode'], configure=run_config)
         return run_ptpython
 
     def get_ptipython(self, options):
@@ -323,7 +363,7 @@ class Command(BaseCommand):
             imported_objects = self.get_imported_objects(options)
             history_filename = os.path.expanduser('~/.ptpython_history')
             embed(user_ns=imported_objects, history_filename=history_filename,
-                  vi_mode=options.get('vi_mode', False), configure=run_config)
+                  vi_mode=options['vi_mode'], configure=run_config)
         return run_ptipython
 
     def set_application_name(self, options):
@@ -360,18 +400,18 @@ class Command(BaseCommand):
 
     @signalcommand
     def handle(self, *args, **options):
-        use_kernel = options.get('kernel', False)
-        use_notebook = options.get('notebook', False)
-        use_ipython = options.get('ipython', False)
-        use_bpython = options.get('bpython', False)
-        use_plain = options.get('plain', False)
-        use_ptpython = options.get('ptpython', False)
-        use_ptipython = options.get('ptipython', False)
-        verbosity = int(options.get('verbosity', 1))
+        use_kernel = options['kernel']
+        use_notebook = options['notebook']
+        use_ipython = options['ipython']
+        use_bpython = options['bpython']
+        use_plain = options['plain']
+        use_ptpython = options['ptpython']
+        use_ptipython = options['ptipython']
+        verbosity = options["verbosity"]
         print_sql = getattr(settings, 'SHELL_PLUS_PRINT_SQL', False)
         truncate = getattr(settings, 'SHELL_PLUS_PRINT_SQL_TRUNCATE', 1000)
 
-        if options.get("print_sql", False) or print_sql:
+        if options["print_sql"] or print_sql:
 
             # Code from http://gist.github.com/118990
             try:
@@ -473,4 +513,7 @@ class Command(BaseCommand):
             print(self.style.ERROR("Could not load %s interactive Python environment." % shell_name))
             return
 
+        if self.tests_mode:
+            return 130
+
         shell()
