@@ -24,9 +24,10 @@ KNOWN ISSUES:
 import importlib
 import sys
 import six
+import argparse
 from typing import Dict, Union, Callable, Optional  # NOQA
 from django.apps import apps
-from django.core.management import BaseCommand, CommandError, sql as _sql
+from django.core.management import BaseCommand, CommandError
 from django.core.management.base import OutputWrapper
 from django.core.management.color import no_style
 from django.db import connection, transaction, models
@@ -134,16 +135,12 @@ class SQLDiff(object):
         self.has_differences = None
         self.app_models = app_models
         self.options = options
-        self.dense = options.get('dense_output', False)
+        self.dense = options['dense_output']
 
-        try:
-            self.introspection = connection.introspection
-        except AttributeError:
-            from django.db import get_introspection_module
-            self.introspection = get_introspection_module()
+        self.introspection = connection.introspection
 
         self.cursor = connection.cursor()
-        self.django_tables = self.get_django_tables(options.get('only_existing', True))
+        self.django_tables = self.introspection.django_table_names(only_existing=options['only_existing'])
         # TODO: We are losing information about tables which are views here
         self.db_tables = [table_info.name for table_info in self.introspection.get_table_list(self.cursor)]
         self.differences = []
@@ -193,18 +190,6 @@ class SQLDiff(object):
         # type: () -> Dict[int, Union[str, Callable]]
         return self.DATA_TYPES_REVERSE_OVERRIDE
 
-    def get_django_tables(self, only_existing):
-        try:
-            django_tables = self.introspection.django_table_names(only_existing=only_existing)
-        except AttributeError:
-            # backwards compatibility for before introspection refactoring (r8296)
-            try:
-                django_tables = _sql.django_table_names(only_existing=only_existing)
-            except AttributeError:
-                # backwards compatibility for before svn r7568
-                django_tables = _sql.django_table_list(only_existing=only_existing)
-        return django_tables
-
     def sql_to_dict(self, query, param):
         """ sql_to_dict(query, param) -> list of dicts
 
@@ -236,11 +221,7 @@ class SQLDiff(object):
             reverse_type = DATA_TYPES_REVERSE_OVERRIDE[type_code]
         else:
             try:
-                try:
-                    reverse_type = self.introspection.data_types_reverse[type_code]
-                except AttributeError:
-                    # backwards compatibility for before introspection refactoring (r8296)
-                    reverse_type = self.introspection.DATA_TYPES_REVERSE.get(type_code)
+                reverse_type = self.introspection.get_field_type(type_code, description)
             except KeyError:
                 reverse_type = self.get_field_db_type_lookup(type_code)
                 if not reverse_type:
@@ -282,6 +263,15 @@ class SQLDiff(object):
         if field and getattr(field, 'geography', False):
             kwargs['geography'] = True
 
+        if reverse_type == 'GeometryField':
+            geo_col = description[0]
+            # Getting a more specific field type and any additional parameters
+            # from the `get_geometry_type` routine for the spatial backend.
+            reverse_type, geo_params = self.introspection.get_geometry_type(table_name, geo_col)
+            if geo_params:
+                kwargs.update(geo_params)
+            reverse_type = 'django.contrib.gis.db.models.fields.%s' % reverse_type
+
         extra_kwargs = self.get_field_db_type_kwargs(kwargs, description, field, table_name, reverse_type)
         kwargs.update(extra_kwargs)
 
@@ -388,12 +378,18 @@ class SQLDiff(object):
         for field_name, field in six.iteritems(fieldmap):
             if field_name not in db_fields:
                 field_output = []
-                if field.rel:
-                    field_output.extend([field.rel.to._meta.db_table, field.rel.to._meta.get_field(field.rel.field_name).column])
+
+                # TODO: Remove 'field.rel' after dropping Django 1.8 support
+                remote_field = field.remote_field if hasattr(field, 'remote_field') else field.rel
+                if remote_field:
+                    remote_field_model = remote_field.model if hasattr(remote_field, 'model') else remote_field.to
+                    field_output.extend([remote_field_model._meta.db_table, remote_field_model._meta.get_field(remote_field.field_name).column])
                     op = 'fkey-missing-in-db'
                 else:
                     op = 'field-missing-in-db'
                 field_output.append(field.db_type(connection=connection))
+                if self.options['include_defaults'] and field.has_default():
+                    field_output.append('DEFAULT %s' % field.get_prep_value(field.get_default()))
                 if not field.null:
                     field_output.append('NOT NULL')
                 self.add_difference(op, table_name, field_name, *field_output)
@@ -472,7 +468,7 @@ class SQLDiff(object):
             table_name = meta.db_table
             app_label = meta.app_label
 
-            if self.options.get('include_proxy_models', False) and meta.proxy:
+            if not self.options['include_proxy_models'] and meta.proxy:
                 continue
 
             if cur_app_label != app_label:
@@ -530,7 +526,7 @@ class SQLDiff(object):
 
     def print_diff(self, style=no_style()):
         """ print differences to stdout """
-        if self.options.get('sql', True):
+        if self.options['sql']:
             self.print_diff_sql(style)
         else:
             self.print_diff_text(style)
@@ -792,12 +788,10 @@ class PostgresqlSQLDiff(SQLDiff):
             1185: lambda: self.get_data_type_arrayfield(base_field='DateTimeField'),
             1231: lambda: self.get_data_type_arrayfield(base_field='DecimalField'),
             # {'name': 'django.contrib.postgres.fields.ArrayField', 'kwargs': {'base_field': 'IntegerField'}},
+            1186: lambda: self.get_data_type_arrayfield(base_field='DurationField'),
+            # 1186: 'django.db.models.fields.DurationField',
+            3614: 'django.contrib.postgres.search.SearchVectorField',
             3802: 'django.contrib.postgres.fields.JSONField',
-            # postgis types (TODO: support is very incomplete)
-            17506: 'django.contrib.gis.db.models.fields.PointField',
-            16392: 'django.contrib.gis.db.models.fields.PointField',
-            55902: 'django.contrib.gis.db.models.fields.MultiPolygonField',
-            16946: 'django.contrib.gis.db.models.fields.MultiPolygonField'
         }
 
     def get_constraints(self, cursor, table_name, introspection):
@@ -1001,25 +995,41 @@ to check/debug ur models compared to the real database tables and columns."""
         parser.add_argument('app_label', nargs='*')
         parser.add_argument(
             '--all-applications', '-a', action='store_true',
+            default=False,
             dest='all_applications',
-            help="Automaticly include all application from INSTALLED_APPS.")
+            help="Automaticly include all application from INSTALLED_APPS."
+        )
         parser.add_argument(
             '--not-only-existing', '-e', action='store_false',
+            default=True,
             dest='only_existing',
-            help="Check all tables that exist in the database, not only "
-            "tables that should exist based on models.")
+            help="Check all tables that exist in the database, not only tables that should exist based on models."
+        )
         parser.add_argument(
             '--dense-output', '-d', action='store_true', dest='dense_output',
-            help="Shows the output in dense format, normally output is "
-            "spreaded over multiple lines.")
+            default=False,
+            help="Shows the output in dense format, normally output is spreaded over multiple lines."
+        )
         parser.add_argument(
             '--output_text', '-t', action='store_false', dest='sql',
             default=True,
-            help="Outputs the differences as descriptive text instead of SQL")
+            help="Outputs the differences as descriptive text instead of SQL"
+        )
         parser.add_argument(
             '--include-proxy-models', action='store_true', dest='include_proxy_models',
             default=False,
-            help="Include proxy models in the graph")
+            help="Include proxy models in the graph"
+        )
+        parser.add_argument(
+            '--include-defaults', action='store_true', dest='include_defaults',
+            default=False,
+            help="Include default values in SQL output (beta feature)"
+        )
+        parser.add_argument(
+            '--migrate-for-tests', action='store_true', dest='migrate_for_tests',
+            default=False,
+            help=argparse.SUPPRESS
+        )
 
     def __init__(self, *args, **kwargs):
         super(Command, self).__init__(*args, **kwargs)
@@ -1028,8 +1038,9 @@ to check/debug ur models compared to the real database tables and columns."""
     @signalcommand
     def handle(self, *args, **options):
         from django.conf import settings
+        from django.core.management import call_command
 
-        app_labels = options.get('app_label')
+        app_labels = options['app_label']
         engine = None
         if hasattr(settings, 'DATABASES'):
             engine = settings.DATABASES['default']['ENGINE']
@@ -1043,7 +1054,7 @@ to check/debug ur models compared to the real database tables and columns."""
 because you haven't specified the DATABASE_ENGINE setting.
 Edit your settings file and change DATABASE_ENGINE to something like 'postgresql' or 'mysql'.""")
 
-        if options.get('all_applications', False):
+        if options['all_applications']:
             app_models = apps.get_models(include_auto_created=True)
         else:
             if not app_labels:
@@ -1060,6 +1071,10 @@ Edit your settings file and change DATABASE_ENGINE to something like 'postgresql
         if not app_models:
             raise CommandError('Unable to execute sqldiff no models founds.')
 
+        migrate_for_tests = options['migrate_for_tests']
+        if migrate_for_tests:
+            call_command("migrate", *app_labels, no_input=True, run_syncdb=True)
+
         if not engine:
             engine = connection.__module__.split('.')[-2]
 
@@ -1077,7 +1092,7 @@ Edit your settings file and change DATABASE_ENGINE to something like 'postgresql
         try:
             super(Command, self).execute(*args, **options)
         except CommandError as e:
-            if options.get('traceback', False):
+            if options['traceback']:
                 raise
 
             # self.stderr is not guaranteed to be set here
